static ConVar sm_outlineball_base_duration;
static int g_NumberOfOutlineEffectsAppliedOnClient[MAXPLAYERS + 1];

#define FLIGHT_TIME_TO_MAX_OUTLINE_DURATION 0.8

void Outline_Init() {
    TF2CustomAttrBall_Register("outline", BallImpactEffect_Outline);
    sm_outlineball_base_duration = CreateConVar("sm_outlineball_base_duration", "15.0", "Outline effect base duration", _, true, 0.0);
}

static void BallImpactEffect_Outline(int target, int projectile, float flightTime, const char[] effectName) {
    bool touched = view_as<bool>(GetEntProp(projectile, Prop_Send, "m_bTouched"));
    if(touched) {
        return;
    }
    SetEntProp(projectile, Prop_Send, "m_bTouched", 1);

    int ownerEntity = GetEntPropEnt(projectile, Prop_Send, "m_hOwnerEntity");
    int launcher = GetEntPropEnt(projectile, Prop_Send, "m_hLauncher");

    float duration = CalcOutlineEffectDuration(flightTime);
    ApplyOutlineEffect(target, ownerEntity, duration);

    float origin[3];
    GetEntPropVector(projectile, Prop_Data, "m_vecAbsOrigin", origin);
    SDKHooks_TakeDamage(target, launcher, ownerEntity, 10.0, .bypassHooks = false);

    CreateTimer(4.0, Timer_RemoveEntity, EntIndexToEntRef(projectile));
}

static Action Timer_RemoveEntity(Handle timer, int projectileRef) {
    int projectile = EntRefToEntIndex(projectileRef);
    if(projectile > 0) {
        RemoveEntity(projectile);
    }

    return Plugin_Handled;
}

static void ApplyOutlineEffect(int targetClient, int launcherClient, float duration) {
    // Spies shouldn't get outlined
    if(TF2_GetPlayerClass(targetClient) == TFClass_Spy) {
        return;
    }

    CreateGlowForClient(targetClient);
    g_NumberOfOutlineEffectsAppliedOnClient[targetClient]++;

    CreateTimer(duration, Timer_RemoveOutline, GetClientUserId(targetClient));
}

static Action Timer_RemoveOutline(Handle timer, int userid) {
    int client = GetClientOfUserId(userid);
    if(client <= 0 || !IsValidEntity(client)) {
        return Plugin_Handled;
    }

    int appliedOutlineCount = g_NumberOfOutlineEffectsAppliedOnClient[client];
    if(appliedOutlineCount <= 0) {
        return Plugin_Handled;
    }

    if(appliedOutlineCount == 1) {
        RemoveOutlineEffect(client);
    }

    g_NumberOfOutlineEffectsAppliedOnClient[client]--;

    return Plugin_Handled;
}

static void RemoveOutlineEffect(int targetClient) {
    RemoveGlow(targetClient);
}

// Copied from sandman
static float CalcOutlineEffectDuration(float flightTime) {
    if(flightTime > FLIGHT_TIME_TO_MAX_OUTLINE_DURATION) {
        flightTime = FLIGHT_TIME_TO_MAX_OUTLINE_DURATION;
    }

    float ratio = flightTime / FLIGHT_TIME_TO_MAX_OUTLINE_DURATION;
    if(ratio <= 0.1) {
        return 0.0;
    }

    float duration = sm_outlineball_base_duration.FloatValue * ratio;

    // TODO: add +2 seconds if it's a critical hit
    // TODO: add +1 second if ratio >= 1

    return duration;
}

static void CreateGlowForClient(int client) {
    int currentGlow = GetGlow(client);
    if(IsValidEntity(currentGlow)) {
        return;
    }

    char clientName[256];
    GetEntPropString(client, Prop_Data, "m_iName", clientName, sizeof(clientName));

    char tempName[256];
    FormatEx(tempName, sizeof(tempName), "outline_attr_%d", client);
    DispatchKeyValue(client, "targetname", tempName);

    int glowEntity = CreateEntityByName("tf_glow");
    if(glowEntity == -1) {
        LogError("Failed to create tf_glow");
        return;
    }

    DispatchKeyValue(glowEntity, "target", tempName);
    DispatchSpawn(glowEntity);
    AcceptEntityInput(glowEntity, "Enable");

    // Reset client name
    SetEntPropString(client, Prop_Data, "m_iName", clientName);

    int color[4];

    TFTeam team = TF2_GetClientTeam(client);
    switch(team) {
        case TFTeam_Red: {
            color = {255, 0, 0, 255};
        }
        case TFTeam_Blue: {
            color = {0, 0, 255, 255};
        }
        default: {
            color = {200, 200, 200, 255};
        }
    }

    SetVariantColor(color);
    AcceptEntityInput(glowEntity, "SetGlowColor");
}

static void RemoveGlow(int entity) {
    int glowEntity = GetGlow(entity);
    if(IsValidEntity(glowEntity)) {
        RemoveEntity(glowEntity);
    }
}

static int GetGlow(int entity) {
    int currentEntity = -1;
    while((currentEntity = FindEntityByClassname(currentEntity, "tf_glow")) != -1) {
        if(GetEntPropEnt(currentEntity, Prop_Send, "m_hTarget") == entity) {
            return currentEntity;
        }
    }

    return -1;
}
