enum StickBombState {
    StickBombState_NotEquipped,
    StickBombState_Normal,
    StickBombState_Exploded,
}

static StickBombState g_StickBombStateCache[MAXPLAYERS + 1] = { StickBombState_NotEquipped, ... };

void StickBomb_Init() {
    HookEvent("post_inventory_application", Event_PostInventoryApplication);
}

static Action Event_PostInventoryApplication(Event event, const char[] name, bool dontBroadcast) {
    int client = GetClientOfUserId(event.GetInt("userid"));
    if(client < 1 || client > MaxClients) {
        return Plugin_Continue;
    }

    int weapon = GetStickBomb(client);
    if(!IsValidEntity(weapon)) {
        return Plugin_Continue;
    }

    UpdateStickBombModel(weapon);

    return Plugin_Continue;
}

void StickBomb_OnClientPutInServer(int client) {
    SDKHook(client, SDKHook_PostThinkPost, OnClientPostThinkPost);
}

static void OnClientPostThinkPost(int client) {
    if(!IsPlayerAlive(client)) {
        return;
    }

    int weapon = GetStickBomb(client);
    if(!IsValidEntity(weapon)) {
        g_StickBombStateCache[client] = StickBombState_NotEquipped;
        return;
    }

    bool detonated = view_as<bool>(GetEntProp(weapon, Prop_Send, "m_iDetonated"));
    if(detonated && g_StickBombStateCache[client] != StickBombState_Exploded) {
        g_StickBombStateCache[client] = StickBombState_Exploded;
        UpdateStickBombModel(weapon);
    } else if(!detonated && g_StickBombStateCache[client] != StickBombState_Normal) {
        g_StickBombStateCache[client] = StickBombState_Normal;
        UpdateStickBombModel(weapon);
    }
}

void StickBomb_OnClientDisconnect(int client) {
    g_StickBombStateCache[client] = StickBombState_NotEquipped;
}

static void UpdateStickBombModel(int weapon) {
    bool detonated = view_as<bool>(GetEntProp(weapon, Prop_Send, "m_iDetonated"));

    char modelPath[PLATFORM_MAX_PATH];
    if(detonated) {
        TF2CustAttr_GetString(weapon, "stick bomb exploded model override", modelPath, sizeof(modelPath));
    } else {
        TF2CustAttr_GetString(weapon, "stick bomb normal model override", modelPath, sizeof(modelPath));
    }

    if(modelPath[0] != '\0') {
        TF2CustAttr_SetString(weapon, "clientmodel override", modelPath);
        int owner = GetEntPropEnt(weapon, Prop_Send, "m_hOwner");
        if(IsValidEntity(owner) && owner > 0 && owner <= MaxClients) {
            ViewModelOverride_UpdateClientWeaponModel(owner);
        }
    }
}

static int GetStickBomb(client) {
    int melee = GetPlayerWeaponSlot(client, TFWeaponSlot_Melee);
    if(!IsValidEntity(melee)) {
        return -1;
    }

    char classname[64];
    GetEntityClassname(melee, classname, sizeof(classname));
    if(!StrEqual(classname, "tf_weapon_stickbomb")) {
        return -1;
    }

    return melee;
}
