#define MAX_UBER_SOUNDS 10

static float g_UberLevelCache[MAXPLAYERS + 1];

void UberSound_Init() {
    HookEvent("player_chargedeployed", Event_PlayerChargeDeployed_Post);
}

void UberSound_OnClientDisconnect(int client) {
    g_UberLevelCache[client] = 0.0;
}

void UberSound_OnClientPutInServer(int client) {
    SDKHook(client, SDKHook_PostThinkPost, OnClientPostThinkPost);
}

static void OnClientPostThinkPost(int client) {
    if(!IsPlayerAlive(client)) {
        g_UberLevelCache[client] = 0.0;
        return;
    }

    int medigun = GetMedigun(client);

    if(!IsValidEntity(medigun)) {
        g_UberLevelCache[client] = 0.0;
        return;
    }

    float chargeLevel = GetEntPropFloat(medigun, Prop_Send, "m_flChargeLevel");
    UpdateChargeLevel(client, g_UberLevelCache[client], chargeLevel);
}

static int GetMedigun(int client) {
    int secondaryWeapon = GetPlayerWeaponSlot(client, TFWeaponSlot_Secondary);
    if(!IsValidEntity(secondaryWeapon)) {
        return -1;
    }

    char classname[64];
    GetEntityClassname(secondaryWeapon, classname, sizeof(classname));
    if(!StrEqual(classname, "tf_weapon_medigun")) {
        return -1;
    }

    return secondaryWeapon;
}

static void UpdateChargeLevel(int client, float oldLevel, float newLevel) {
    if(oldLevel != newLevel && newLevel == 1.0) {
        OnUberReady(client);
    }

    g_UberLevelCache[client] = newLevel;
}

static void OnUberReady(int client) {
    int secondaryWeapon = GetPlayerWeaponSlot(client, TFWeaponSlot_Secondary);
    if(!IsValidEntity(secondaryWeapon)) {
        return;
    }

    char uberReadySoundPath[PLATFORM_MAX_PATH];
    TF2CustAttr_GetString(secondaryWeapon, "uber ready sound", uberReadySoundPath, sizeof(uberReadySoundPath));
    if(uberReadySoundPath[0] == '\0') {
        return;
    }

    PrecacheSound(uberReadySoundPath);
    EmitSoundToClient(client, uberReadySoundPath, .channel = SNDCHAN_STATIC);
}

static Action Event_PlayerChargeDeployed_Post(Event event, const char[] name, bool dontBroadcast) {
    int client = GetClientOfUserId(event.GetInt("userid"));
    if(client == 0) {
        return Plugin_Continue;
    }

    PlayUberSound(client);

    return Plugin_Continue;
}

// TODO: stop sound on death
static void PlayUberSound(int client) {
    int medigun = GetPlayerWeaponSlot(client, TFWeaponSlot_Secondary);
    if(medigun == -1) {
        return;
    }

    char soundPath[PLATFORM_MAX_PATH];
    if(!GetRandomUberSound(medigun, soundPath, sizeof(soundPath))) {
        return;
    }

    PrecacheSound(soundPath);
    EmitSoundToAll(soundPath, client, .channel = SNDCHAN_STATIC);
}

static bool GetRandomUberSound(int medigun, char[] returnedSoundPath, int maxLength) {
    char attributeValue[4096];
    TF2CustAttr_GetString(medigun, "uber sound", attributeValue, sizeof(attributeValue));
    if(attributeValue[0] == '\0') {
        return false;
    }

    char sounds[MAX_UBER_SOUNDS][PLATFORM_MAX_PATH];

    int soundCount = ExplodeString(attributeValue, ";", sounds, sizeof(sounds), sizeof(sounds[]));
    if(soundCount == 0) {
        return false;
    }

    int randomIndex = GetRandomInt(0, soundCount - 1);
    strcopy(returnedSoundPath, maxLength, sounds[randomIndex]);
    return true;
}
